@@ -5,7 +5,7 @@ describe "NestedAttributes plugin" do
     if should.is_a?(Array)
       should.should include(is)
     else
-      should.should == is
+      is.should == should
     end
   end
 
@@ -36,11 +36,16 @@ describe "NestedAttributes plugin" do
     @Artist.one_to_many :albums, :class=>@Album, :key=>:artist_id
     @Artist.one_to_many :concerts, :class=>@Concert, :key=>:artist_id
     @Artist.one_to_one :first_album, :class=>@Album, :key=>:artist_id
+    @Artist.one_to_one :first_concert, :class=>@Concert, :key=>:artist_id
+    @Concert.one_to_many :albums, :class=>@Album, :key=>:artist_id, :primary_key=>:artist_id
     @Album.many_to_one :artist, :class=>@Artist, :reciprocal=>:albums
     @Album.many_to_many :tags, :class=>@Tag, :left_key=>:album_id, :right_key=>:tag_id, :join_table=>:at
+    @Tag.many_to_many :albums, :class=>@Album, :left_key=>:tag_id, :right_key=>:album_id, :join_table=>:at
     @Artist.nested_attributes :albums, :first_album, :destroy=>true, :remove=>true
     @Artist.nested_attributes :concerts, :destroy=>true, :remove=>true
     @Album.nested_attributes :artist, :tags, :destroy=>true, :remove=>true
+    @Artist.nested_attributes :first_concert
+    @Concert.nested_attributes :albums
     @db.sqls
   end
   
@@ -71,6 +76,91 @@ describe "NestedAttributes plugin" do
       ["INSERT INTO albums (artist_id, name) VALUES (1, 'Al')", "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)"])
   end
   
+  it "should support creating new one_to_many and one_to_one objects with presence validations on the foreign key" do
+    @Album.class_eval do
+      plugin :validation_helpers
+      def validate
+        validates_integer :artist_id
+        super
+      end
+    end
+    a = @Artist.new({:name=>'Ar', :albums_attributes=>[{:name=>'Al'}]})
+    @db.sqls.should == []
+    a.save
+    check_sql_array("INSERT INTO artists (name) VALUES ('Ar')",
+      ["INSERT INTO albums (artist_id, name) VALUES (1, 'Al')", "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)"])
+
+    a = @Artist.new(:name=>'Ar')
+    a.id = 1
+    a.first_album_attributes = {:name=>'Al'}
+    @db.sqls.should == []
+    a.save
+    check_sql_array(["INSERT INTO artists (name, id) VALUES ('Ar', 1)", "INSERT INTO artists (id, name) VALUES (1, 'Ar')"],
+      "UPDATE albums SET artist_id = NULL WHERE (artist_id = 1)",
+      ["INSERT INTO albums (artist_id, name) VALUES (1, 'Al')", "INSERT INTO albums (name, artist_id) VALUES ('Al', 1)"])
+  end
+
+  it "should support creating new one_to_many and one_to_one objects with composite keys with presence validations on the foreign key" do
+    insert = nil
+    @Album.class_eval do
+      plugin :validation_helpers
+      def validate
+        validates_integer :artist_id
+        super
+      end
+    end
+    @Concert.class_eval do
+      define_method :_insert do
+        insert = values.dup
+      end
+      def before_create # Have to define the CPK somehow.
+        self.tour = 'To'
+        self.date = '2004-04-05'
+        super
+      end
+      def after_create
+        super
+        self.artist_id = 3
+      end
+    end
+
+    c = @Concert.new(:playlist=>'Pl')
+    @db.sqls.should == []
+    c.albums_attributes = [{:name=>'Al'}]
+    c.save
+    insert.should == {:tour=>'To', :date=>'2004-04-05', :playlist=>'Pl'}
+    check_sql_array(["INSERT INTO albums (name, artist_id) VALUES ('Al', 3)", "INSERT INTO albums (artist_id, name) VALUES (3, 'Al')"])
+
+    @Concert.class_eval do
+      plugin :validation_helpers
+      def validate
+        validates_integer :artist_id
+        super
+      end
+    end
+
+    a = @Artist.new(:name=>'Ar')
+    a.id = 1
+    a.first_concert_attributes = {:playlist=>'Pl'}
+    @db.sqls.should == []
+    a.save
+    check_sql_array(["INSERT INTO artists (name, id) VALUES ('Ar', 1)", "INSERT INTO artists (id, name) VALUES (1, 'Ar')"],
+                   "UPDATE concerts SET artist_id = NULL WHERE (artist_id = 1)")
+    insert.should == {:tour=>'To', :date=>'2004-04-05', :artist_id=>1, :playlist=>'Pl'}
+  end
+
+  it "should should not remove existing values from object when validating" do
+    @Artist.one_to_one :first_album, :class=>@Album, :key=>:id
+    @Artist.nested_attributes :first_album
+    @db.fetch = {:id=>1}
+    a = @Artist.load(:id=>1)
+    a.set(:first_album_attributes=>{:id=>1, :name=>'Ar'})
+    a.first_album.values.should == {:id=>1, :name=>'Ar'}
+    @db.sqls.should == ["SELECT * FROM albums WHERE (albums.id = 1) LIMIT 1"]
+    a.save_changes
+    check_sql_array("UPDATE albums SET name = 'Ar' WHERE (id = 1)")
+  end
+
   it "should support creating new many_to_many objects" do
     a = @Album.new({:name=>'Al', :tags_attributes=>[{:name=>'T'}]})
     @db.sqls.should == []
@@ -81,16 +171,19 @@ describe "NestedAttributes plugin" do
   end
   
   it "should add new objects to the cached association array as soon as the *_attributes= method is called" do
-    a = @Artist.new({:name=>'Ar', :albums_attributes=>[{:name=>'Al', :tags_attributes=>[{:name=>'T'}]}]})
+    a = @Artist.new({:name=>'Ar', :first_album_attributes=>{:name=>'B'}, :albums_attributes=>[{:name=>'Al', :tags_attributes=>[{:name=>'T'}]}]})
     a.albums.should == [@Album.new(:name=>'Al')]
+    a.albums.first.artist.should == a
     a.albums.first.tags.should == [@Tag.new(:name=>'T')]
+    a.first_album.should == @Album.new(:name=>'B')
+    a.first_album.artist.should == a
   end
   
   it "should support creating new objects with composite primary keys" do
     insert = nil
     @Concert.class_eval do
       define_method :_insert do
-        insert = values
+        insert = values.dup
       end
       def before_create # Have to define the CPK somehow.
         self.tour = 'To'
@@ -111,7 +204,7 @@ describe "NestedAttributes plugin" do
     @Album.class_eval do
       unrestrict_primary_key
       define_method :_insert do
-        insert = values
+        insert = values.dup
       end
     end
     a = @Artist.new({:name=>'Ar', :albums_attributes=>[{:id=>7, :name=>'Al'}]})
@@ -126,7 +219,7 @@ describe "NestedAttributes plugin" do
     @Artist.nested_attributes :concerts, :unmatched_pk=>:create
     @Concert.class_eval do
       define_method :_insert do
-        insert = values
+        insert = values.dup
       end
     end
     a = @Artist.new({:name=>'Ar', :concerts_attributes=>[{:tour=>'To', :date=>'2004-04-05', :playlist=>'Pl'}]})
@@ -231,6 +324,7 @@ describe "NestedAttributes plugin" do
     ar = @Artist.load(:id=>20, :name=>'Ar')
     ar.associations[:albums] = [al]
     ar.set(:albums_attributes=>[{:id=>10, :_remove=>'t'}])
+    ar.associations[:albums].should == []
     @db.sqls.should == []
     @Album.dataset._fetch = {:id=>1}
     ar.save
@@ -244,6 +338,7 @@ describe "NestedAttributes plugin" do
     t = @Tag.load(:id=>20, :name=>'T')
     a.associations[:tags] = [t]
     a.set(:tags_attributes=>[{:id=>20, :_remove=>true}])
+    a.associations[:tags].should == []
     @db.sqls.should == []
     a.save
     @db.sqls.should == ["DELETE FROM at WHERE ((album_id = 10) AND (tag_id = 20))", "UPDATE albums SET name = 'Al' WHERE (id = 10)"]
@@ -340,7 +435,7 @@ describe "NestedAttributes plugin" do
     @Album.nested_attributes :artist
     proc{a.set(:artist_attributes=>{:id=>'20', :_delete=>'1'})}.should raise_error(Sequel::Error)
     @Album.nested_attributes :artist, :destroy=>true
-    proc{a.set(:artist_attributes=>{:id=>'20', :_delete=>'1'})}.should_not raise_error(Sequel::Error)
+    proc{a.set(:artist_attributes=>{:id=>'20', :_delete=>'1'})}.should_not raise_error
   end
   
   it "should only allow removing associated objects if :remove option is used in the nested_attributes call" do
@@ -350,7 +445,7 @@ describe "NestedAttributes plugin" do
     @Album.nested_attributes :artist
     proc{a.set(:artist_attributes=>{:id=>'20', :_remove=>'1'})}.should raise_error(Sequel::Error)
     @Album.nested_attributes :artist, :remove=>true
-    proc{a.set(:artist_attributes=>{:id=>'20', :_remove=>'1'})}.should_not raise_error(Sequel::Error)
+    proc{a.set(:artist_attributes=>{:id=>'20', :_remove=>'1'})}.should_not raise_error
   end
   
   it "should raise an Error if a primary key is given in a nested attribute hash, but no matching associated object exists" do
@@ -358,7 +453,7 @@ describe "NestedAttributes plugin" do
     ar = @Artist.load(:id=>20, :name=>'Ar')
     ar.associations[:albums] = [al]
     proc{ar.set(:albums_attributes=>[{:id=>30, :_delete=>'t'}])}.should raise_error(Sequel::Error)
-    proc{ar.set(:albums_attributes=>[{:id=>10, :_delete=>'t'}])}.should_not raise_error(Sequel::Error)
+    proc{ar.set(:albums_attributes=>[{:id=>10, :_delete=>'t'}])}.should_not raise_error
   end
   
   it "should not raise an Error if an unmatched primary key is given, if the :strict=>false option is used" do
@@ -388,7 +483,7 @@ describe "NestedAttributes plugin" do
     co = @Concert.load(:tour=>'To', :date=>'2004-04-05', :playlist=>'Pl')
     ar.associations[:concerts] = [co]
     proc{ar.set(:concerts_attributes=>[{:tour=>'To', :date=>'2004-04-04', :_delete=>'t'}])}.should raise_error(Sequel::Error)
-    proc{ar.set(:concerts_attributes=>[{:tour=>'To', :date=>'2004-04-05', :_delete=>'t'}])}.should_not raise_error(Sequel::Error)
+    proc{ar.set(:concerts_attributes=>[{:tour=>'To', :date=>'2004-04-05', :_delete=>'t'}])}.should_not raise_error
   end
 
   it "should not raise an Error if an unmatched composite primary key is given, if the :strict=>false option is used" do
@@ -572,4 +667,31 @@ describe "NestedAttributes plugin" do
     proc{al.set(:tags_attributes=>[{:id=>30, :name=>'T2', :number=>3}])}.should raise_error(Sequel::Error)
     proc{al.set(:tags_attributes=>[{:name=>'T2', :number=>3}])}.should raise_error(Sequel::Error)
   end
+
+  it "should allow per-call options via the set_nested_attributes method" do
+    @Tag.columns :id, :name, :number
+    @Album.nested_attributes :tags
+
+    al = @Album.load(:id=>10, :name=>'Al')
+    t = @Tag.load(:id=>30, :name=>'T', :number=>10)
+    al.associations[:tags] = [t]
+    al.set_nested_attributes(:tags, [{:id=>30, :name=>'T2'}, {:name=>'T3'}], :fields=>[:name])
+    @db.sqls.should == []
+    al.save
+    check_sql_array("UPDATE albums SET name = 'Al' WHERE (id = 10)",
+      "UPDATE tags SET name = 'T2' WHERE (id = 30)",
+      "INSERT INTO tags (name) VALUES ('T3')",
+      ["INSERT INTO at (album_id, tag_id) VALUES (10, 1)", "INSERT INTO at (tag_id, album_id) VALUES (1, 10)"])
+    proc{al.set_nested_attributes(:tags, [{:id=>30, :name=>'T2', :number=>3}], :fields=>[:name])}.should raise_error(Sequel::Error)
+    proc{al.set_nested_attributes(:tags, [{:name=>'T2', :number=>3}], :fields=>[:name])}.should raise_error(Sequel::Error)
+  end
+
+  it "should have set_nested_attributes method raise error if called with a bad association" do
+    proc{@Album.load(:id=>10, :name=>'Al').set_nested_attributes(:tags2, [{:id=>30, :name=>'T2', :number=>3}], :fields=>[:name])}.should raise_error(Sequel::Error)
+  end
+
+  it "should have set_nested_attributes method raise error if called with an association that doesn't support nested attributes" do
+    @Tag.columns :id, :name, :number
+    proc{@Album.load(:id=>10, :name=>'Al').set_nested_attributes(:tags, [{:id=>30, :name=>'T2', :number=>3}], :fields=>[:name])}.should raise_error(Sequel::Error)
+  end
 end
